from unittest import TestCase
from tgt_grease.core import GreaseContainer, Configuration
from tgt_grease.enterprise.Model import Scan, PrototypeConfig, BaseSourceClass
from datetime import datetime
from bson.objectid import ObjectId
import json
import platform
import time
import uuid


class TestSource(BaseSourceClass):
    def parse_source(self, configuration):
        self._data = [
            {
                'field': 'var',
                'field1': 'var1',
                'field2': 'var2',
                'field3': 'var3',
                'field4': 'var4',
                'field5': 'var5',
            },
            {
                'field': 'var',
                'field1': 'var1',
                'field2': 'var2',
                'field3': 'var3',
                'field4': 'var4',
                'field5': 'var5',
            },
            {
                'field': 'var',
                'field1': 'var1',
                'field2': 'var2',
                'field3': 'var3',
                'field4': 'var4',
                'field5': 'var5',
            },
            {
                'field': 'var',
                'field1': 'var1',
                'field2': 'var2',
                'field3': 'var3',
                'field4': 'var4',
                'field5': 'var5',
            },
            {
                'field': str(uuid.uuid4()),
                'field1': str(uuid.uuid4()),
                'field2': str(uuid.uuid4()),
                'field3': str(uuid.uuid4()),
                'field4': str(uuid.uuid4()),
                'field5': str(uuid.uuid4())
            },
            {
                'field': 'var',
                'field1': 'var1',
                'field2': 'var2',
                'field3': 'var3',
                'field4': 'var4',
                'field5': 'var5',
            },
            {
                'field': 'var',
                'field1': 'var1',
                'field2': 'var2',
                'field3': 'var3',
                'field4': 'var4',
                'field5': 'var5',
            },
            {
                'field': 'var',
                'field1': 'var1',
                'field2': 'var2',
                'field3': 'var3',
                'field4': 'var4',
                'field5': 'var5',
            },
            {
                'field': str(uuid.uuid4()),
                'field1': str(uuid.uuid4()),
                'field2': str(uuid.uuid4()),
                'field3': str(uuid.uuid4()),
                'field4': str(uuid.uuid4()),
                'field5': str(uuid.uuid4())
            },
            {
                'field': str(uuid.uuid4()),
                'field1': str(uuid.uuid4()),
                'field2': str(uuid.uuid4()),
                'field3': str(uuid.uuid4()),
                'field4': str(uuid.uuid4()),
                'field5': str(uuid.uuid4())
            },
            {
                'field': str(uuid.uuid4()),
                'field1': str(uuid.uuid4()),
                'field2': str(uuid.uuid4()),
                'field3': str(uuid.uuid4()),
                'field4': str(uuid.uuid4()),
                'field5': str(uuid.uuid4())
            }
        ]
        return True

    def mock_data(self, configuration):
        self._data = [
            {
                'field': 'var',
                'field1': 'var1',
                'field2': 'var2',
                'field3': 'var3',
                'field4': 'var4',
                'field5': 'var5',
            },
            {
                'field': 'var',
                'field1': 'var1',
                'field2': 'var2',
                'field3': 'var3',
                'field4': 'var4',
                'field5': 'var5',
            },
            {
                'field': 'var',
                'field1': 'var1',
                'field2': 'var2',
                'field3': 'var3',
                'field4': 'var4',
                'field5': 'var5',
            },
            {
                'field': 'var',
                'field1': 'var1',
                'field2': 'var2',
                'field3': 'var3',
                'field4': 'var4',
                'field5': 'var5',
            },
            {
                'field': str(uuid.uuid4()),
                'field1': str(uuid.uuid4()),
                'field2': str(uuid.uuid4()),
                'field3': str(uuid.uuid4()),
                'field4': str(uuid.uuid4()),
                'field5': str(uuid.uuid4())
            },
            {
                'field': 'var',
                'field1': 'var1',
                'field2': 'var2',
                'field3': 'var3',
                'field4': 'var4',
                'field5': 'var5',
            },
            {
                'field': 'var',
                'field1': 'var1',
                'field2': 'var2',
                'field3': 'var3',
                'field4': 'var4',
                'field5': 'var5',
            },
            {
                'field': 'var',
                'field1': 'var1',
                'field2': 'var2',
                'field3': 'var3',
                'field4': 'var4',
                'field5': 'var5',
            },
            {
                'field': str(uuid.uuid4()),
                'field1': str(uuid.uuid4()),
                'field2': str(uuid.uuid4()),
                'field3': str(uuid.uuid4()),
                'field4': str(uuid.uuid4()),
                'field5': str(uuid.uuid4())
            },
            {
                'field': str(uuid.uuid4()),
                'field1': str(uuid.uuid4()),
                'field2': str(uuid.uuid4()),
                'field3': str(uuid.uuid4()),
                'field4': str(uuid.uuid4()),
                'field5': str(uuid.uuid4())
            },
            {
                'field': str(uuid.uuid4()),
                'field1': str(uuid.uuid4()),
                'field2': str(uuid.uuid4()),
                'field3': str(uuid.uuid4()),
                'field4': str(uuid.uuid4()),
                'field5': str(uuid.uuid4())
            }
        ]


class TestScan(TestCase):

    def test_scan(self):
        # setup
        configList = [
            {
                "name": "test1",
                "job": "fakeJob",
                "exe_env": "windows",
                "source": "TestSource",
                "logic": {
                    "regex": [
                        {
                            "field": "character",
                            "pattern": ".*skywalker.*"
                        }
                    ]
                }
            }
        ]
        ioc = GreaseContainer()
        ioc.ensureRegistration()
        ioc.getConfig().set('trace', True, 'Logging')
        ioc.getConfig().set('verbose', True, 'Logging')
        fil = open(ioc.getConfig().greaseConfigFile, 'r')
        data = json.loads(fil.read())
        fil.close()
        fil = open(ioc.getConfig().greaseConfigFile, 'w')
        data['Import']['searchPath'].append('tgt_grease.enterprise.Model.tests')
        fil.write(json.dumps(data, sort_keys=True, indent=4))
        fil.close()
        Configuration.ReloadConfig()
        jServer = ioc.getCollection('JobServer')
        jID1 = jServer.insert_one({
                'jobs': 0,
                'os': platform.system().lower(),
                'roles': ["general"],
                'prototypes': ["detect"],
                'active': True,
                'activationTime': datetime.utcnow()
        }).inserted_id
        time.sleep(1)
        jID2 = jServer.insert_one({
                'jobs': 0,
                'os': platform.system().lower(),
                'roles': ["general"],
                'prototypes': ["detect"],
                'active': True,
                'activationTime': datetime.utcnow()
        }).inserted_id

        # Begin Test
        conf = PrototypeConfig(ioc)
        conf.load(reloadConf=True, ConfigurationList=configList)
        scanner = Scan(ioc)
        # Scan Environment
        self.assertTrue(scanner.Parse())
        # Begin ensuring environment is how we expect
        # we assert less or equal because sometimes uuid's are close :p
        self.assertLessEqual(ioc.getCollection('SourceData').find({
            'detectionServer': ObjectId(jID1)
        }).count(), 3)
        self.assertLessEqual(ioc.getCollection('SourceData').find({
            'detectionServer': ObjectId(jID2)
        }).count(), 3)
        self.assertLessEqual(ioc.getCollection('JobServer').find_one({
            '_id': ObjectId(jID1)
        })['jobs'], 3)
        self.assertLessEqual(ioc.getCollection('JobServer').find_one({
            '_id': ObjectId(jID2)
        })['jobs'], 3)

        # clean up
        fil = open(ioc.getConfig().greaseConfigFile, 'r')
        data = json.loads(fil.read())
        fil.close()
        # remove collection
        ioc.getCollection('TestProtoType').drop()
        # remove prototypes
        data['NodeInformation']['ProtoTypes'] = []
        # pop search path
        trash = data['Import']['searchPath'].pop()
        # close out
        fil = open(ioc.getConfig().greaseConfigFile, 'w')
        fil.write(json.dumps(data, sort_keys=True, indent=4))
        fil.close()
        jServer.delete_one({'_id': ObjectId(jID1)})
        jServer.delete_one({'_id': ObjectId(jID2)})
        ioc.getCollection('SourceData').drop()
        ioc.getCollection('Dedup_Sourcing').drop()
        ioc.getConfig().set('trace', False, 'Logging')
        ioc.getConfig().set('verbose', False, 'Logging')
        Configuration.ReloadConfig()

    def test_config_load_empty(self):
        scan = Scan()
        confs = scan.generate_config_set()
        self.assertTrue(len(confs) == 0)
        self.assertTrue(isinstance(confs, list))

    def test_config_load_all(self):
        conf = PrototypeConfig()
        scan = Scan(conf.ioc)
        conf.load(reloadConf=True)
        configList = [
            {
                "name": "test1",
                "job": "fakeJob",
                "exe_env": "windows",
                "source": "swapi",
                "logic": {
                    "regex": [
                        {
                            "field": "character",
                            "pattern": ".*skywalker.*"
                        }
                    ]
                }
            },
            {
                "name": "test2",
                "job": "fakeJob",
                "exe_env": "windows",
                "source": "Google",
                "logic": {
                    "regex": [
                        {
                            "field": "character",
                            "pattern": ".*skywalker.*"
                        }
                    ]
                }
            },
            {
                "name": "test3",
                "job": "fakeJob",
                "exe_env": "windows",
                "source": "stackOverflow",
                "logic": {
                    "regex": [
                        {
                            "field": "character",
                            "pattern": ".*skywalker.*"
                        }
                    ],
                    "exists": [
                        {
                            "field": "var"
                        }
                    ]
                }
            }
        ]
        self.assertTrue(isinstance(conf.load(ConfigurationList=configList), dict))
        configs = scan.generate_config_set()
        self.assertTrue(isinstance(configs, list))
        self.assertTrue(len(configs) == len(configList))
        conf.load(reloadConf=True)

    def test_config_load_source(self):
        conf = PrototypeConfig()
        scan = Scan(conf.ioc)
        conf.load(reloadConf=True)
        configList = [
            {
                "name": "test1",
                "job": "fakeJob",
                "exe_env": "windows",
                "source": "swapi",
                "logic": {
                    "regex": [
                        {
                            "field": "character",
                            "pattern": ".*skywalker.*"
                        }
                    ]
                }
            },
            {
                "name": "test2",
                "job": "fakeJob",
                "exe_env": "windows",
                "source": "Google",
                "logic": {
                    "regex": [
                        {
                            "field": "character",
                            "pattern": ".*skywalker.*"
                        }
                    ]
                }
            },
            {
                "name": "test3",
                "job": "fakeJob",
                "exe_env": "windows",
                "source": "stackOverflow",
                "logic": {
                    "regex": [
                        {
                            "field": "character",
                            "pattern": ".*skywalker.*"
                        }
                    ],
                    "exists": [
                        {
                            "field": "var"
                        }
                    ]
                }
            }
        ]
        self.assertTrue(isinstance(conf.load(ConfigurationList=configList), dict))
        configs = scan.generate_config_set(source='Google')
        self.assertTrue(isinstance(configs, list))
        self.assertTrue(len(configs) == 1)
        conf.load(reloadConf=True)

    def test_config_load_name(self):
        conf = PrototypeConfig()
        scan = Scan(conf.ioc)
        conf.load(reloadConf=True)
        configList = [
            {
                "name": "test1",
                "job": "fakeJob",
                "exe_env": "windows",
                "source": "swapi",
                "logic": {
                    "regex": [
                        {
                            "field": "character",
                            "pattern": ".*skywalker.*"
                        }
                    ]
                }
            },
            {
                "name": "test2",
                "job": "fakeJob",
                "exe_env": "windows",
                "source": "Google",
                "logic": {
                    "regex": [
                        {
                            "field": "character",
                            "pattern": ".*skywalker.*"
                        }
                    ]
                }
            },
            {
                "name": "test3",
                "job": "fakeJob",
                "exe_env": "windows",
                "source": "stackOverflow",
                "logic": {
                    "regex": [
                        {
                            "field": "character",
                            "pattern": ".*skywalker.*"
                        }
                    ],
                    "exists": [
                        {
                            "field": "var"
                        }
                    ]
                }
            }
        ]
        self.assertTrue(isinstance(conf.load(ConfigurationList=configList), dict))
        configs = scan.generate_config_set(config='test1')
        self.assertTrue(isinstance(configs, list))
        self.assertTrue(len(configs) == 1)
        conf.load(reloadConf=True)

    def test_config_load_source_and_name(self):
        conf = PrototypeConfig()
        scan = Scan(conf.ioc)
        conf.load(reloadConf=True)
        configList = [
            {
                "name": "test1",
                "job": "fakeJob",
                "exe_env": "windows",
                "source": "swapi",
                "logic": {
                    "regex": [
                        {
                            "field": "character",
                            "pattern": ".*skywalker.*"
                        }
                    ]
                }
            },
            {
                "name": "test2",
                "job": "fakeJob",
                "exe_env": "windows",
                "source": "Google",
                "logic": {
                    "regex": [
                        {
                            "field": "character",
                            "pattern": ".*skywalker.*"
                        }
                    ]
                }
            },
            {
                "name": "test3",
                "job": "fakeJob",
                "exe_env": "windows",
                "source": "stackOverflow",
                "logic": {
                    "regex": [
                        {
                            "field": "character",
                            "pattern": ".*skywalker.*"
                        }
                    ],
                    "exists": [
                        {
                            "field": "var"
                        }
                    ]
                }
            }
        ]
        self.assertTrue(isinstance(conf.load(ConfigurationList=configList), dict))
        configs = scan.generate_config_set(config='test1', source='swapi')
        self.assertTrue(isinstance(configs, list))
        self.assertTrue(len(configs) == 1)
        conf.load(reloadConf=True)

    def test_config_load_bad_source_good_name(self):
        conf = PrototypeConfig()
        scan = Scan(conf.ioc)
        conf.load(reloadConf=True)
        configList = [
            {
                "name": "test1",
                "job": "fakeJob",
                "exe_env": "windows",
                "source": "swapi",
                "logic": {
                    "regex": [
                        {
                            "field": "character",
                            "pattern": ".*skywalker.*"
                        }
                    ]
                }
            },
            {
                "name": "test2",
                "job": "fakeJob",
                "exe_env": "windows",
                "source": "Google",
                "logic": {
                    "regex": [
                        {
                            "field": "character",
                            "pattern": ".*skywalker.*"
                        }
                    ]
                }
            },
            {
                "name": "test3",
                "job": "fakeJob",
                "exe_env": "windows",
                "source": "stackOverflow",
                "logic": {
                    "regex": [
                        {
                            "field": "character",
                            "pattern": ".*skywalker.*"
                        }
                    ],
                    "exists": [
                        {
                            "field": "var"
                        }
                    ]
                }
            }
        ]
        self.assertTrue(isinstance(conf.load(ConfigurationList=configList), dict))
        configs = scan.generate_config_set(config='test1', source='Google')
        self.assertTrue(isinstance(configs, list))
        self.assertTrue(len(configs) == 0)
        conf.load(reloadConf=True)

    def test_config_load_good_source_bad_name(self):
        conf = PrototypeConfig()
        scan = Scan(conf.ioc)
        conf.load(reloadConf=True)
        configList = [
            {
                "name": "test1",
                "job": "fakeJob",
                "exe_env": "windows",
                "source": "swapi",
                "logic": {
                    "regex": [
                        {
                            "field": "character",
                            "pattern": ".*skywalker.*"
                        }
                    ]
                }
            },
            {
                "name": "test2",
                "job": "fakeJob",
                "exe_env": "windows",
                "source": "Google",
                "logic": {
                    "regex": [
                        {
                            "field": "character",
                            "pattern": ".*skywalker.*"
                        }
                    ]
                }
            },
            {
                "name": "test3",
                "job": "fakeJob",
                "exe_env": "windows",
                "source": "stackOverflow",
                "logic": {
                    "regex": [
                        {
                            "field": "character",
                            "pattern": ".*skywalker.*"
                        }
                    ],
                    "exists": [
                        {
                            "field": "var"
                        }
                    ]
                }
            }
        ]
        self.assertTrue(isinstance(conf.load(ConfigurationList=configList), dict))
        configs = scan.generate_config_set(config='test7', source='swapi')
        self.assertTrue(isinstance(configs, list))
        self.assertTrue(len(configs) == 0)
        conf.load(reloadConf=True)



from tgt_grease.enterprise.Model import Detector
import re


class Regex(Detector):
    """Regular Expression Detector for GREASE Detection

    A Typical Regex configuration looks like this::

        {
            ...
            'logic': {
                'Regex': [
                    {
                        'field': String, # <-- Field to search for
                        'pattern': String, # <-- Regex to perform on field
                        'variable': Boolean, # <-- OPTIONAL, if true then create a context variable of result
                        'variable_name: String # <-- REQUIRED IF variable, name of context variable
                    }
                    ...
                ]
                ...
            }
        }

    """

    def processObject(self, source, ruleConfig):
        """Processes an object and returns valid rule data

        Data returned in the second parameter from this method should be in this form::

            {
                '<field>': Object # <-- if specified as a variable then return the key->Value pairs
                ...
            }

        Args:
            source (dict): Source Data
            ruleConfig (list[dict]): Rule Configuration Data

        Return:
            tuple: first element boolean for success; second dict for any fields returned as variables

        """
        final = {}
        finalBool = False
        if not isinstance(source, dict):
            return False, {}
        if not isinstance(ruleConfig, list):
            return False, {}
        else:
            # loop through configuration for each set of logical configurations
            for block in ruleConfig:
                if not isinstance(block, dict):
                    self.ioc.getLogger().error(
                        "INVALID REGEX LOGICAL BLOCK! NOT TYPE LIST [{0}]".format(str(type(block))),
                        notify=False
                    )
                    return False, {}
                else:
                    # look for field and perform regex
                    if block.get('field') in source:
                        if source.get(block.get('field')):
                            result = re.findall(block.get('pattern'), str(source.get(block.get('field'))))
                            if len(result):
                                finalBool = True
                                if block.get('variable') and block.get('variable_name'):
                                    final[str(block.get('variable_name'))] = result
                                else:
                                    continue
                            else:
                                self.ioc.getLogger().trace(
                                    "Field did not pass regex",
                                    verbose=True
                                )
                                return False, {}
                        else:
                            # truthy false field value
                            self.ioc.getLogger().trace(
                                "Field [{0}] equates to false [{1}]".format(
                                    block.get('field'),
                                    source.get(block.get('field'))
                                ),
                                notify=False,
                                verbose=True
                            )
                            return False, {}
                    else:
                        self.ioc.getLogger().trace(
                            "Field not found in source [{0}]".format(block.get('field')),
                            notify=False,
                            verbose=True
                        )
                        return False, {}
            return finalBool, final



import importlib
from tgt_grease.core import Logging


class ImportTool(object):
    """Import Tooling for getting instances of classes automatically

    Attributes:
        _log (Logging): Logger for the class

    """

    _log = None

    def __init__(self, logger):
        if not isinstance(logger, Logging):
            raise Exception("Invalid Constructor, logger element not of type Logging class")
        else:
            self._log = logger

    def load(self, className):
        """Dynamic loading of classes for the system

        Args:
            className (str): Class name to search for

        Returns:
            object: If an object is found it is returned
            None: If an object is not found and error occurs None is returned

        """
        self._log.trace("Attempting to load class [{0}]".format(className), trace=True)
        for path in self._log.getConfig().get('Import', 'searchPath'):
            self._log.trace("Searching path [{0}]".format(path), trace=True)
            try:
                SearchModule = importlib.import_module(str(path))
            except ImportError:
                self._log.error("Failed to import module [{0}]".format(path), verbose=True)
                continue
            if className in dir(SearchModule):
                if className.startswith("__"):
                    # No need to try magic methods
                    continue
                req = getattr(SearchModule, str(className))
                try:
                    instance = req()
                    return instance
                except AttributeError:
                    self._log.error(
                        "ATTRERROR: Failed to create instance of class [{0}] from module [{1}]".format(className, path),
                        verbose=True
                    )
                    continue
                except TypeError:
                    self._log.error(
                        "TYPEERROR: Failed to create instance of class [{0}] from module [{1}]".format(className, path),
                        verbose=True
                    )
        return None



from tgt_grease.core import GreaseContainer
import pkg_resources
import os
import fnmatch
import json

GREASE_PROTOTYPE_CONFIGURATION = None


class PrototypeConfig(object):
    """Responsible for Scanning/Detection/Scheduling configuration

    Structure of Configuration::

        {
            'configuration': {
                'pkg': [], # <-- Loaded from pkg_resources.resource_filename('tgt_grease.enterprise.Model', 'config/')
                'fs': [], # <-- Loaded from `<GREASE_DIR>/etc/*.config.json`
                'mongo': [] # <-- Loaded from the Configuration Mongo Collection
            },
            'raw': [], # <-- All loaded configurations
            'sources': [], # <-- list of sources found in configurations
            'source': {} # <-- keys will be source values list of configs for that source
            'names': [], # <-- all configs via their name so to allow dialing
            'name': {} # <-- all configs via their name so to allow being dialing
        }

    Structure of a configuration file::

        {
            "name": String,
            "job": String,
            "exe_env": String, # <-- If not provided will be default as 'general'
            "source": String,
            "logic": {
                # I need to be the logical blocks for Detection
            }
        }

    Attributes:
        ioc (GreaseContainer): IOC access

    """

    def __init__(self, ioc=None):
        global GREASE_PROTOTYPE_CONFIGURATION
        if ioc and isinstance(ioc, GreaseContainer):
            self.ioc = ioc
        else:
            self.ioc = GreaseContainer()
        if not GREASE_PROTOTYPE_CONFIGURATION:
            GREASE_PROTOTYPE_CONFIGURATION = self.load()

    def getConfiguration(self):
        """Returns the Configuration Object loaded into memory

        Returns:
            dict: Configuration object

        """
        global GREASE_PROTOTYPE_CONFIGURATION
        if not GREASE_PROTOTYPE_CONFIGURATION:
            self.load(reloadConf=True)
        return GREASE_PROTOTYPE_CONFIGURATION

    def load(self, reloadConf=False, ConfigurationList=None):
        """[Re]loads configuration data about the current execution node

        Configuration data loads from 3 places in GREASE. The first is internal to the package, if one were to manually
        add their own files into the package in the current directory following the file pattern. The next is following
        the same pattern but loaded from `<GREASE_DIR>/etc/`. The final place GREASE looks for configuration data is
        from the `configuration` collection in MongoDB

        Args:
            reloadConf (bool): If True this will reload the global object. False will return the object
            ConfigurationList (list of dict): If provided will load the list of dict for config after validation

        Note:
            Providing a configuration *automatically* reloads the memory structure of prototype configuration

        Returns:
            dict: Current Configuration information

        """
        global GREASE_PROTOTYPE_CONFIGURATION
        if ConfigurationList:
            conf = dict()
            conf['configuration'] = dict()
            conf['configuration']['ConfigurationList'] = self.validate_config_list(ConfigurationList)
            conf['raw'] = conf['configuration']['ConfigurationList']
            # split by configuration sets
            # the list of configured sources
            conf['sources'] = list()
            # the actual configurations for each source
            conf['source'] = dict()
            # configurations to get via name
            conf['names'] = list()
            # the actual configurations for each config name
            conf['name'] = dict()
            for config in conf.get('raw'):  # type: dict
                if config.get('source') in conf['sources']:
                    conf['source'][config.get('source')].append(config)
                else:
                    conf['sources'].append(config.get('source'))
                    conf['source'][config.get('source')] = list()
                    conf['source'][config.get('source')].append(config)
                if config.get('name') in conf['names']:
                    self.ioc.getLogger().error(
                        "Prototype Configuration [{0}] already found! Overwriting".format(config.get('name'))
                    )
                    conf['name'][config.get('name')] = config
                else:
                    conf['names'].append(config.get('name'))
                    conf['name'][config.get('name')] = config
            GREASE_PROTOTYPE_CONFIGURATION = conf
            return conf
        # fill out raw results
        conf = dict()
        conf['configuration'] = dict()
        conf['raw'] = []
        pkg = self.validate_config_list(self.load_from_fs(
            pkg_resources.resource_filename('tgt_grease.enterprise.Model', 'config/')
        ))
        for newConfig in pkg:
            conf['raw'].append(newConfig)
        conf['configuration']['pkg'] = pkg
        del pkg
        fs = self.validate_config_list(self.load_from_fs(
            self.ioc.getConfig().get('Configuration', 'dir')
        ))
        for newConfig in fs:
            conf['raw'].append(newConfig)
        conf['configuration']['fs'] = fs
        del fs
        mongo = self.validate_config_list(self.load_from_mongo())
        for newConfig in mongo:
            conf['raw'].append(newConfig)
        conf['configuration']['mongo'] = mongo
        del mongo
        # split by configuration sets
        # the list of configured sources
        conf['sources'] = list()
        # the actual configurations for each source
        conf['source'] = dict()
        # configurations to get via name
        conf['names'] = list()
        # the actual configurations for each config name
        conf['name'] = dict()
        for config in conf.get('raw'):  # type: dict
            if config.get('source') in conf['sources']:
                conf['source'][config.get('source')].append(config)
            else:
                conf['sources'].append(config.get('source'))
                conf['source'][config.get('source')] = list()
                conf['source'][config.get('source')].append(config)
            if config.get('name') in conf['names']:
                self.ioc.getLogger().error(
                    "Prototype Configuration [{0}] already found! Overwriting".format(config.get('name'))
                )
                conf['name'][config.get('name')] = config
            else:
                conf['names'].append(config.get('name'))
                conf['name'][config.get('name')] = config
        # return block
        if not reloadConf:
            return conf
        else:
            GREASE_PROTOTYPE_CONFIGURATION = conf
            return conf

    def get_sources(self):
        """Returns the list of sources to be scanned

        Returns:
            list: List of sources

        """
        global GREASE_PROTOTYPE_CONFIGURATION  # type: dict
        if GREASE_PROTOTYPE_CONFIGURATION:
            return GREASE_PROTOTYPE_CONFIGURATION.get('sources', [])
        else:
            self.ioc.getLogger().error("GREASE Prototype configuration is not loaded", trace=True, notify=False)
            return []

    def get_source(self, name):
        """Get all configuration by source by name

        Args:
            name (str): Source name to get

        Returns:
            list[dict]: Configuration if found else empty dict

        """
        global GREASE_PROTOTYPE_CONFIGURATION
        if GREASE_PROTOTYPE_CONFIGURATION:
            return GREASE_PROTOTYPE_CONFIGURATION.get('source').get(name, [])
        else:
            self.ioc.getLogger().error("GREASE Prototype configuration not loaded", notify=False, trace=True)
            return []

    def get_names(self):
        """Returns the list of names of configs

        Returns:
            list: List of config names

        """
        global GREASE_PROTOTYPE_CONFIGURATION  # type: dict
        if GREASE_PROTOTYPE_CONFIGURATION:
            return GREASE_PROTOTYPE_CONFIGURATION.get('names', [])
        else:
            self.ioc.getLogger().error("GREASE Prototype configuration is not loaded", trace=True, notify=False)
            return []

    def get_config(self, name):
        """Get Configuration by name

        Args:
            name (str): Configuration name to get

        Returns:
            dict: Configuration if found else empty dict

        """
        global GREASE_PROTOTYPE_CONFIGURATION
        if GREASE_PROTOTYPE_CONFIGURATION:
            if GREASE_PROTOTYPE_CONFIGURATION.get('name'):
                return GREASE_PROTOTYPE_CONFIGURATION.get('name').get(name, {})
            else:
                self.ioc.getLogger().error("GREASE Configuration Not Found", notify=False, trace=True)
                return {}
        else:
            self.ioc.getLogger().error("GREASE Prototype configuration not loaded", notify=False, trace=True)
            return {}

    def load_from_fs(self, directory):
        """Loads configurations from provided directory

        Note:
            Pattern is `*.config.json`

        Args:
            directory (str): Directory to load from

        Returns:
            list of dict: configurations

        """
        self.ioc.getLogger().trace("Loading Configurations from directory [{0}]".format(directory), trace=True)
        intermediate = list()
        matches = []
        for root, dirnames, filenames in os.walk(directory):
            for filename in fnmatch.filter(filenames, '*.config.json'):
                matches.append(os.path.join(root, filename))
        for doc in matches:
            self.ioc.getLogger().trace("Attempting to load [{0}]".format(doc), trace=True)
            with open(doc, 'rb') as current_file:
                content = current_file.read()
                if isinstance(content, bytes):
                    content = content.decode()
            try:
                intermediate.append(json.loads(content))
                self.ioc.getLogger().trace("Successfully loaded [{0}]".format(doc), trace=True)
            except ValueError:
                self.ioc.getLogger().error("Failed to load [{0}]".format(doc), trace=True, notify=False)
                continue
        self.ioc.getLogger().trace("total documents returned from fs [{0}]".format(len(intermediate)), trace=True)
        return intermediate

    def load_from_mongo(self):
        """Returns all active configurations from the mongo collection Configuration

        Structure of Configuration expected in Mongo::

            {
                "name": String,
                "job": String,
                "exe_env": String, # <-- If not provided will be default as 'general'
                "active": Boolean, # <-- set to true to load configuration
                "type": "prototype_config", # <-- MUST BE THIS VALUE; For it is the config type :)
                "source": String,
                "logic": {
                    # I need to be the logical blocks for Detection
                }
            }

        Returns:
            list of dict: Configurations

        """
        self.ioc.getLogger().trace("Loading Configurations from mongo", trace=True)
        mConf = []
        for conf in self.ioc.getCollection('Configuration').find({
            'active': True,
            'type': 'prototype_config'
        }):
            mConf.append(dict(conf))
        self.ioc.getLogger().trace("total documents returned from mongo [{0}]".format(len(mConf)), trace=True)
        return mConf

    def validate_config_list(self, configs):
        """Validates a configuration List

        Args:
            configs (list[dict]): Configuration List

        Returns:
            list: The Valid configurations

        """
        final = []
        self.ioc.getLogger().trace("Total configurations to validate [{0}]".format(len(configs)))
        for conf in configs:
            if self.validate_config(conf):
                final.append(conf)
        return final

    def validate_config(self, config):
        """Validates a configuration

        The default JSON Schema is this::

            {
                "name": String,
                "job": String,
                "exe_env": String, # <-- If not provided will be default as 'general'
                "source": String,
                "logic": {
                    # I need to be the logical blocks for Detection
                }
            }

        Args:
            config (dict): Configuration to validate

        Returns:
            bool: If it is a valid configuration

        """
        self.ioc.getLogger().trace("Configuration to be validated: [{0}]".format(config), trace=True)
        if not isinstance(config, dict):
            self.ioc.getLogger().error(
                "Configuration Validation Failed! Not of Type Dict::Got [{0}]".format(str(type(config))),
                trace=True,
                notify=False
            )
        if config.get('name'):
            if not isinstance(config.get('name'), str):
                config['name'] = str(config.get('name'))
        else:
            self.ioc.getLogger().error("Configuration does not have valid name field", trace=True, notify=False)
            return False
        if config.get('job'):
            if not isinstance(config.get('job'), str):
                config['job'] = str(config.get('job'))
        else:
            self.ioc.getLogger().error("Configuration does not have valid job field", trace=True, notify=False)
            return False
        if config.get('source'):
            if not isinstance(config.get('source'), str):
                config['source'] = str(config.get('source'))
        else:
            self.ioc.getLogger().error("Configuration does not have valid source field", trace=True, notify=False)
            return False
        if not isinstance(config.get('logic'), dict):
            self.ioc.getLogger().error("Configuration does not have valid logic field", trace=True, notify=False)
            return False
        if not config.get('logic'):
            # empty dictionary check AKA no logical blocks
            return False
        for key, params in config.get('logic').items():
            if not isinstance(params, list):
                self.ioc.getLogger().error("Configuration logic field was not list!", trace=True, notify=False)
                return False
            for block in params:
                if not isinstance(block, dict):
                    self.ioc.getLogger().error("Configuration logical block was not dict", trace=True, notify=False)
                    return False
        return True



from unittest import TestCase
from tgt_grease.core import Configuration
import os


class TestConfiguration(TestCase):
    def test_config_is_false_before_startup(self):
        self.assertFalse(Configuration.get('MongoDB', 'host'))

    def test_initialization(self):
        conf = Configuration()
        self.assertTrue(conf.get('Connectivity', 'MongoDB'))

    def test_filesystem(self):
        conf = Configuration()
        for elem in conf.FileSystem:
            self.assertTrue(os.path.isdir(conf.greaseDir + conf.fs_sep + elem))

    def test_no_key(self):
        conf = Configuration()
        self.assertDictEqual({'MongoDB': {'host': 'localhost', 'port': 27017}}, conf.get('Connectivity'))

    def test_default(self):
        conf = Configuration()
        self.assertTrue(conf.get('FakeSection', default=True))
        self.assertTrue(conf.get('Connectivity', 'MongoDB'))

    def test_get_section(self):
        conf = Configuration()
        # test for section that should have stuff
        self.assertTrue(conf.get('Notifications'))
        # Section that is empty
        self.assertFalse(conf.get('Additional'))

    def test_defaults(self):
        conf = Configuration()
        self.assertEqual(conf.get('Connectivity', 'MongoDB').get('host'), 'localhost')
        self.assertEqual(conf.get('Connectivity', 'MongoDB').get('port'), 27017)
        self.assertEqual(conf.get('Logging', 'mode'), 'filesystem')
        self.assertEqual(conf.get('Logging', 'verbose'), False)
        self.assertEqual(conf.get('Logging', 'file'), conf.greaseDir + 'log' + conf.fs_sep + 'grease.log')
        self.assertEqual(conf.get('Configuration', 'dir'), conf.greaseDir + 'etc' + conf.fs_sep)
        self.assertEqual(conf.get('Sourcing', 'dir'), conf.greaseDir + 'etc' + conf.fs_sep)

    def test_set(self):
        conf = Configuration()
        self.assertFalse(conf.get('Logging', 'trace'))
        conf.set('trace', True, 'Logging')
        self.assertTrue(conf.get('Logging', 'trace'))
        conf.set('trace', False, 'Logging')
        self.assertFalse(conf.get('Logging', 'trace'))



from unittest import TestCase
from tgt_grease.enterprise.Model import Detect, Deduplication, PrototypeConfig
from bson.objectid import ObjectId
import platform
import datetime


class TestDetect(TestCase):

    def test_get_schedule_empty(self):
        d = Detect()
        self.assertFalse(d.getScheduledSource())

    def test_get_schedule_staged(self):
        d = Detect()
        d.ioc.getCollection('SourceData').insert_one({
                    'grease_data': {
                        'sourcing': {
                            'server': ObjectId(d.ioc.getConfig().NodeIdentity)
                        },
                        'detection': {
                            'server': ObjectId(d.ioc.getConfig().NodeIdentity),
                            'detectionStart': None,
                            'detectionEnd': None,
                            'detection': {}
                        },
                        'scheduling': {
                            'schedulingServer': None,
                            'schedulingStart': None,
                            'schedulingEnd': None
                        },
                        'execution': {
                            'server': None,
                            'assignmentTime': None,
                            'executionStart': None,
                            'executionEnd': None,
                            'context': {},
                            'executionSuccess': False,
                            'commandSuccess': False,
                            'failures': 0,
                            'retryTime': datetime.datetime.utcnow()
                        }
                    },
                    'source': str('test').encode('utf-8'),
                    'configuration': str('testConfig').encode('utf-8'),
                    'data': {'dev': 'test'},
                    'createTime': datetime.datetime.utcnow(),
                    'expiry': Deduplication.generate_max_expiry_time(1)
        })
        self.assertTrue(d.getScheduledSource())
        self.assertEqual(
            d.getScheduledSource().get('grease_data').get('detection').get('server'),
            ObjectId(d.ioc.getConfig().NodeIdentity)
        )
        d.ioc.getCollection('SourceData').drop()

    def test_detection(self):
        d = Detect()
        source = {
            'key': 'var',
            'ver': 'key',
            'greg': 'old'
        }
        configuration = {
            'name': 'demo config',
            'job': 'otherThing',
            'exe_env': 'general',
            'source': 'Google',
            'logic': {
                'Regex': [
                    {
                        'field': 'key',
                        'pattern': '.*',
                        'variable': True,
                        'variable_name': 'field'
                    },
                    {
                        'field': 'ver',
                        'pattern': '.*'
                    }
                ]
            }
        }
        result, resultData = d.detection(source, configuration)
        self.assertTrue(result)
        self.assertTrue(resultData.get('field'))

    def test_detection_bad_source(self):
        d = Detect()
        configuration = {
            'name': 'demo config',
            'job': 'otherThing',
            'exe_env': 'general',
            'source': 'Google',
            'logic': {
                'Regex': [
                    {
                        'field': 'key',
                        'pattern': '.*',
                        'variable': True,
                        'variable_name': 'field'
                    },
                    {
                        'field': 'ver',
                        'pattern': '.*'
                    }
                ]
            }
        }
        result, resultData = d.detection([], configuration)
        self.assertFalse(result)
        self.assertFalse(resultData)

    def test_detection_bad_config(self):
        d = Detect()
        source = {
            'key': 'var',
            'ver': 'key',
            'greg': 'old'
        }
        result, resultData = d.detection(source, [])
        self.assertFalse(result)
        self.assertFalse(resultData)

    def test_detection_failed(self):
        d = Detect()
        source = {
            'key': 'var',
            'ver': 'key',
            'greg': 'old'
        }
        configuration = {
            'name': 'demo config',
            'job': 'otherThing',
            'exe_env': 'general',
            'source': 'Google',
            'logic': {
                'Regex': [
                    {
                        'field': 'ke',
                        'pattern': '.*',
                        'variable': True,
                        'variable_name': 'field'
                    },
                    {
                        'field': 'var',
                        'pattern': '.*'
                    }
                ]
            }
        }
        result, resultData = d.detection(source, configuration)
        self.assertFalse(result)
        self.assertFalse(resultData)

    def test_detection_bad_detector(self):
        d = Detect()
        source = {
            'key': 'var',
            'ver': 'key',
            'greg': 'old'
        }
        configuration = {
            'name': 'demo config',
            'job': 'otherThing',
            'exe_env': 'general',
            'source': 'Google',
            'logic': {
                'regex': [
                    {
                        'field': 'key',
                        'pattern': '.*',
                        'variable': True,
                        'variable_name': 'field'
                    },
                    {
                        'field': 'ver',
                        'pattern': '.*'
                    }
                ]
            }
        }
        result, resultData = d.detection(source, configuration)
        self.assertFalse(result)
        self.assertFalse(resultData)

    def test_detection_bad_logical_block(self):
        d = Detect()
        source = {
            'key': 'var',
            'ver': 'key',
            'greg': 'old'
        }
        configuration = {
            'name': 'demo config',
            'job': 'otherThing',
            'exe_env': 'general',
            'source': 'Google',
            'logic': {
                'Regex': {}
            }
        }
        result, resultData = d.detection(source, configuration)
        self.assertFalse(result)
        self.assertFalse(resultData)

    def test_detection_full(self):
        d = Detect()
        p = PrototypeConfig(d.ioc)
        source = {
            'key': 'var',
            'ver': 'key',
            'greg': 'old'
        }
        configuration = {
            'name': 'demoConfig',
            'job': 'otherThing',
            'exe_env': 'general',
            'source': 'Google',
            'logic': {
                'Regex': [
                    {
                        'field': 'key',
                        'pattern': '.*',
                        'variable': True,
                        'variable_name': 'field'
                    },
                    {
                        'field': 'ver',
                        'pattern': '.*'
                    }
                ]
            }
        }
        p.load(True, [configuration])
        sourceId = d.ioc.getCollection('SourceData').insert_one({
                    'grease_data': {
                        'sourcing': {
                            'server': ObjectId(d.ioc.getConfig().NodeIdentity)
                        },
                        'detection': {
                            'server': ObjectId(d.ioc.getConfig().NodeIdentity),
                            'detectionStart': None,
                            'detectionEnd': None,
                            'detection': {}
                        },
                        'scheduling': {
                            'schedulingServer': None,
                            'schedulingStart': None,
                            'schedulingEnd': None
                        },
                        'execution': {
                            'server': None,
                            'assignmentTime': None,
                            'executionStart': None,
                            'executionEnd': None,
                            'context': {},
                            'executionSuccess': False,
                            'commandSuccess': False,
                            'failures': 0,
                            'retryTime': datetime.datetime.utcnow()
                        }
                    },
                    'source': str('test').encode('utf-8'),
                    'configuration': configuration.get('name'),
                    'data': source,
                    'createTime': datetime.datetime.utcnow(),
                    'expiry': Deduplication.generate_max_expiry_time(1)
        }).inserted_id
        scheduleServer = d.ioc.getCollection('JobServer').insert_one({
                'jobs': 0,
                'os': platform.system().lower(),
                'roles': ["general"],
                'prototypes': ["schedule"],
                'active': True,
                'activationTime': datetime.datetime.utcnow()
        }).inserted_id
        self.assertTrue(d.detectSource())
        d.ioc.getCollection('JobServer').delete_one({'_id': ObjectId(scheduleServer)})
        d.ioc.getCollection('SourceData').delete_one({'_id': ObjectId(sourceId)})

    def test_detection_logic_not_pass(self):
        d = Detect()
        p = PrototypeConfig(d.ioc)
        source = {
            'key': 'var',
            'ver': 'key',
            'greg': 'old'
        }
        configuration = {
            'name': 'demoConfig',
            'job': 'otherThing',
            'exe_env': 'general',
            'source': 'Google',
            'logic': {
                'Regex': [
                    {
                        'field': 'key',
                        'pattern': '.*',
                        'variable': True,
                        'variable_name': 'field'
                    },
                    {
                        'field': 'missingField',
                        'pattern': '.*'
                    }
                ]
            }
        }
        p.load(True, [configuration])
        sourceId = d.ioc.getCollection('SourceData').insert_one({
                    'grease_data': {
                        'sourcing': {
                            'server': ObjectId(d.ioc.getConfig().NodeIdentity)
                        },
                        'detection': {
                            'server': ObjectId(d.ioc.getConfig().NodeIdentity),
                            'detectionStart': None,
                            'detectionEnd': None,
                            'detection': {}
                        },
                        'scheduling': {
                            'schedulingServer': None,
                            'schedulingStart': None,
                            'schedulingEnd': None
                        },
                        'execution': {
                            'server': None,
                            'assignmentTime': None,
                            'executionStart': None,
                            'executionEnd': None,
                            'context': {},
                            'executionSuccess': False,
                            'commandSuccess': False,
                            'failures': 0,
                            'retryTime': datetime.datetime.utcnow()
                        }
                    },
                    'source': str('test').encode('utf-8'),
                    'configuration': configuration.get('name'),
                    'data': source,
                    'createTime': datetime.datetime.utcnow(),
                    'expiry': Deduplication.generate_max_expiry_time(1)
        }).inserted_id
        scheduleServer = d.ioc.getCollection('JobServer').insert_one({
                'jobs': 0,
                'os': platform.system().lower(),
                'roles': ["general"],
                'prototypes': ["schedule"],
                'active': True,
                'activationTime': datetime.datetime.utcnow()
        }).inserted_id
        self.assertTrue(d.detectSource())
        d.ioc.getCollection('JobServer').delete_one({'_id': ObjectId(scheduleServer)})
        d.ioc.getCollection('SourceData').delete_one({'_id': ObjectId(sourceId)})

    def test_detection_full_multi_detector(self):
        d = Detect()
        p = PrototypeConfig(d.ioc)
        source = {
            'key': 'var',
            'ver': 'key',
            'greg': 'old',
            'created': '2017-11-24'
        }
        configuration = {
            'name': 'demoConfig',
            'job': 'otherThing',
            'exe_env': 'general',
            'source': 'Google',
            'logic': {
                'DateRange': [
                    {
                        'field': 'created',
                        'format': '%Y-%m-%d',
                        'min': '2017-11-23',
                        'max': '2017-11-25'
                    }
                ],
                'Regex': [
                    {
                        'field': 'key',
                        'pattern': '.*',
                        'variable': True,
                        'variable_name': 'field'
                    },
                    {
                        'field': 'ver',
                        'pattern': '.*'
                    }
                ],
                'Exists': [
                    {
                        'field': 'greg',
                        'variable': True,
                        'variable_name': 'greg'
                    }
                ]
            }
        }
        p.load(True, [configuration])
        sourceId = d.ioc.getCollection('SourceData').insert_one({
                    'grease_data': {
                        'sourcing': {
                            'server': ObjectId(d.ioc.getConfig().NodeIdentity)
                        },
                        'detection': {
                            'server': ObjectId(d.ioc.getConfig().NodeIdentity),
                            'detectionStart': None,
                            'detectionEnd': None,
                            'detection': {}
                        },
                        'scheduling': {
                            'schedulingServer': None,
                            'schedulingStart': None,
                            'schedulingEnd': None
                        },
                        'execution': {
                            'server': None,
                            'assignmentTime': None,
                            'executionStart': None,
                            'executionEnd': None,
                            'context': {},
                            'executionSuccess': False,
                            'commandSuccess': False,
                            'failures': 0,
                            'retryTime': datetime.datetime.utcnow()
                        }
                    },
                    'source': str('test').encode('utf-8'),
                    'configuration': configuration.get('name'),
                    'data': source,
                    'createTime': datetime.datetime.utcnow(),
                    'expiry': Deduplication.generate_max_expiry_time(1)
        }).inserted_id
        scheduleServer = d.ioc.getCollection('JobServer').insert_one({
                'jobs': 0,
                'os': platform.system().lower(),
                'roles': ["general"],
                'prototypes': ["schedule"],
                'active': True,
                'activationTime': datetime.datetime.utcnow()
        }).inserted_id
        self.assertTrue(d.detectSource())
        self.assertFalse(d.getScheduledSource())
        d.ioc.getCollection('JobServer').delete_one({'_id': ObjectId(scheduleServer)})
        d.ioc.getCollection('SourceData').delete_one({'_id': ObjectId(sourceId)})

    def test_detection_full_multi_detector_fail(self):
        d = Detect()
        p = PrototypeConfig(d.ioc)
        source = {
            'key': 'var',
            'ver': 'key',
            'greg': 'old',
            'created': '2017-11-30'
        }
        configuration = {
            'name': 'demoConfig',
            'job': 'otherThing',
            'exe_env': 'general',
            'source': 'Google',
            'logic': {
                'DateRange': [
                    {
                        'field': 'created',
                        'format': '%Y-%m-%d',
                        'min': '2017-11-23',
                        'max': '2017-11-25'
                    }
                ],
                'Regex': [
                    {
                        'field': 'key',
                        'pattern': '.*',
                        'variable': True,
                        'variable_name': 'field'
                    },
                    {
                        'field': 'ver',
                        'pattern': '.*'
                    }
                ],
                'Exists': [
                    {
                        'field': 'gary'
                    }
                ]
            }
        }
        p.load(True, [configuration])
        sourceId = d.ioc.getCollection('SourceData').insert_one({
                    'grease_data': {
                        'sourcing': {
                            'server': ObjectId(d.ioc.getConfig().NodeIdentity)
                        },
                        'detection': {
                            'server': ObjectId(d.ioc.getConfig().NodeIdentity),
                            'detectionStart': None,
                            'detectionEnd': None,
                            'detection': {}
                        },
                        'scheduling': {
                            'schedulingServer': None,
                            'schedulingStart': None,
                            'schedulingEnd': None
                        },
                        'execution': {
                            'server': None,
                            'assignmentTime': None,
                            'executionStart': None,
                            'executionEnd': None,
                            'context': {},
                            'executionSuccess': False,
                            'commandSuccess': False,
                            'failures': 0,
                            'retryTime': datetime.datetime.utcnow()
                        }
                    },
                    'source': str('test').encode('utf-8'),
                    'configuration': configuration.get('name'),
                    'data': source,
                    'createTime': datetime.datetime.utcnow(),
                    'expiry': Deduplication.generate_max_expiry_time(1)
        }).inserted_id
        scheduleServer = d.ioc.getCollection('JobServer').insert_one({
                'jobs': 0,
                'os': platform.system().lower(),
                'roles': ["general"],
                'prototypes': ["schedule"],
                'active': True,
                'activationTime': datetime.datetime.utcnow()
        }).inserted_id
        self.assertTrue(d.detectSource())
        item = d.ioc.getCollection('SourceData').find_one({
            '_id': ObjectId(sourceId)
        })
        # Will be false since it failed detection
        self.assertFalse(item['grease_data']['scheduling']['schedulingServer'])
        d.ioc.getCollection('JobServer').delete_one({'_id': ObjectId(scheduleServer)})
        d.ioc.getCollection('SourceData').delete_one({'_id': ObjectId(sourceId)})



from unittest import TestCase
from tgt_grease.management.Model import NodeMonitoring
from tgt_grease.enterprise.Model import Deduplication
from tgt_grease.enterprise.Model import PrototypeConfig
from bson.objectid import ObjectId
import datetime
import platform


class TestNodeMonitoring(TestCase):

    def test_get_servers(self):
        n = NodeMonitoring()
        server = n.ioc.getCollection('JobServer').insert_one({
            'jobs': 10,
            'os': platform.system().lower(),
            'roles': [],
            'prototypes': [],
            'active': True,
            'activationTime': datetime.datetime.utcnow()
        }).inserted_id
        result = n.getServers()
        self.assertGreaterEqual(len(result), 1)
        self.assertEqual(
            n.ioc.getCollection('JobServer').delete_one({'_id': server}).deleted_count,
            1
        )

    def test_get_servers_inactive_server(self):
        n = NodeMonitoring()
        server = n.ioc.getCollection('JobServer').insert_one({
            'jobs': 10,
            'os': platform.system().lower(),
            'roles': [],
            'prototypes': [],
            'active': False,
            'activationTime': datetime.datetime.utcnow()
        }).inserted_id
        result = n.getServers()
        self.assertLessEqual(len(result), 1)
        self.assertEqual(
            n.ioc.getCollection('JobServer').delete_one({'_id': server}).deleted_count,
            1
        )
        n.ioc.getCollection('ServerHealth').drop()

    def test_monitor(self):
        n = NodeMonitoring()
        p = PrototypeConfig(n.ioc)
        server1 = n.ioc.getCollection('JobServer').insert_one({
            'jobs': 9,
            'os': platform.system().lower(),
            'roles': [],
            'prototypes': [],
            'active': True,
            'activationTime': datetime.datetime.utcnow()
        }).inserted_id
        server2 = n.ioc.getCollection('JobServer').insert_one({
            'jobs': 9,
            'os': platform.system().lower(),
            'roles': ['test'],
            'prototypes': ['detect', 'schedule'],
            'active': True,
            'activationTime': datetime.datetime.utcnow()
        }).inserted_id
        config = n.ioc.getCollection('Configuration').insert_one(
            {
                'active': True,
                'type': 'prototype_config',
                "name": "test",
                "job": "help",
                "exe_env": "test",
                "source": "test",
                "logic": {
                    "Regex": [
                        {
                            "field": "url",
                            "pattern": ".*",
                            'variable': True,
                            'variable_name': 'url'
                        }
                    ],
                    'Range': [
                        {
                            'field': 'status_code',
                            'min': 199,
                            'max': 201
                        }
                    ]
                },
                'constants': {
                    'test': 'ver'
                }
            }
        ).inserted_id
        p.load(reloadConf=True)
        source = n.ioc.getCollection('SourceData').insert_one({
            'grease_data': {
                'sourcing': {
                    'server': server1
                },
                'detection': {
                    'server': server1,
                    'start': datetime.datetime.utcnow(),
                    'end': datetime.datetime.utcnow(),
                    'detection': {}
                },
                'scheduling': {
                    'server': server1,
                    'start': datetime.datetime.utcnow(),
                    'end': datetime.datetime.utcnow(),
                },
                'execution': {
                    'server': server1,
                    'assignmentTime': datetime.datetime.utcnow(),
                    'completeTime': None,
                    'returnData': {},
                    'executionSuccess': False,
                    'commandSuccess': False,
                    'failures': 0
                }
            },
            'source': 'test',
            'configuration': 'test',
            'createTime': datetime.datetime.utcnow(),
            'expiry': Deduplication.generate_max_expiry_time(1)
        }).inserted_id
        self.assertTrue(n.monitor())
        self.assertEqual(
            n.ioc.getCollection('JobServer').delete_one({'_id': server1}).deleted_count,
            1
        )
        self.assertEqual(
            n.ioc.getCollection('JobServer').delete_one({'_id': server2}).deleted_count,
            1
        )
        n.ioc.getCollection('SourceData').delete_one({'_id': source})
        n.ioc.getCollection('Configuration').delete_one({'_id': config})
        n.ioc.getCollection('ServerHealth').drop()

    def test_scanComplete(self):
        n = NodeMonitoring()
        originalServer = n.ioc.getCollection('JobServer').find_one({'_id': ObjectId(n.ioc.getConfig().NodeIdentity)})
        self.assertTrue(originalServer)
        originalServer = dict(originalServer)
        originalSourcing = n.ioc.getCollection('SourceData').find(
            {'grease_data.execution.server': ObjectId(originalServer.get('_id'))}
        ).count()
        n.scanComplete()
        newSourcing = n.ioc.getCollection('SourceData').find(
            {'grease_data.execution.server': ObjectId(originalServer.get('_id'))}
        ).count()
        newServer = n.ioc.getCollection('JobServer').find_one({'_id': ObjectId(n.ioc.getConfig().NodeIdentity)})
        self.assertTrue(newServer)
        newServer = dict(newServer)
        self.assertGreaterEqual(newSourcing, originalSourcing)
        self.assertGreaterEqual(newServer.get('jobs'), originalServer.get('jobs'))
        n.ioc.getCollection('SourceData').drop()
        n.ioc.getCollection('JobServer').update_one(
            {'_id': ObjectId(originalServer.get('_id'))},
            {
                '$set': {
                    'jobs': originalServer.get('jobs', 0)
                }
            }
        )

    def test_serverAliveNewServerGood(self):
        n = NodeMonitoring()
        server = n.ioc.getCollection('JobServer').insert_one({
            'jobs': 10,
            'os': platform.system().lower(),
            'roles': [],
            'prototypes': [],
            'active': False,
            'activationTime': datetime.datetime.utcnow()
        }).inserted_id
        self.assertTrue(n.serverAlive(str(server)))
        self.assertEqual(
            n.ioc.getCollection('JobServer').delete_one({'_id': server}).deleted_count,
            1
        )
        n.ioc.getCollection('ServerHealth').drop()

    def test_serverAliveCurrentServerGood(self):
        n = NodeMonitoring()
        server = n.ioc.getCollection('JobServer').insert_one({
            'jobs': 10,
            'os': platform.system().lower(),
            'roles': [],
            'prototypes': [],
            'active': False,
            'activationTime': datetime.datetime.utcnow()
        }).inserted_id
        n.ioc.getCollection('ServerHealth').insert_one(
            {
                'server': server,
                'jobs': 9,
                'checkTime': datetime.datetime.utcnow()
            }
        )
        self.assertTrue(n.serverAlive(str(server)))
        self.assertEqual(n.ioc.getCollection('ServerHealth').find({
            'server': server,
            'jobs': 10
        }).count(), 1)
        self.assertEqual(
            n.ioc.getCollection('JobServer').delete_one({'_id': server}).deleted_count,
            1
        )
        n.ioc.getCollection('ServerHealth').drop()

    def test_serverAliveCurrentServerDegraded(self):
        n = NodeMonitoring()
        server = n.ioc.getCollection('JobServer').insert_one({
            'jobs': 9,
            'os': platform.system().lower(),
            'roles': [],
            'prototypes': [],
            'active': False,
            'activationTime': datetime.datetime.utcnow()
        }).inserted_id
        n.ioc.getCollection('ServerHealth').insert_one(
            {
                'server': server,
                'jobs': 9,
                'checkTime': datetime.datetime.utcnow() - datetime.timedelta(minutes=9)
            }
        )
        self.assertTrue(n.serverAlive(str(server)))
        self.assertEqual(n.ioc.getCollection('ServerHealth').find({
            'server': server,
            'jobs': 9
        }).count(), 1)
        self.assertEqual(
            n.ioc.getCollection('JobServer').delete_one({'_id': server}).deleted_count,
            1
        )
        n.ioc.getCollection('ServerHealth').drop()

    def test_serverAliveCurrentServerDead(self):
        n = NodeMonitoring()
        server = n.ioc.getCollection('JobServer').insert_one({
            'jobs': 9,
            'os': platform.system().lower(),
            'roles': [],
            'prototypes': [],
            'active': False,
            'activationTime': datetime.datetime.utcnow()
        }).inserted_id
        n.ioc.getCollection('ServerHealth').insert_one(
            {
                'server': server,
                'jobs': 9,
                'checkTime': datetime.datetime.utcnow() - datetime.timedelta(minutes=10, seconds=1)
            }
        )
        self.assertFalse(n.serverAlive(str(server)))
        self.assertEqual(n.ioc.getCollection('ServerHealth').find({
            'server': server,
            'jobs': 9
        }).count(), 1)
        self.assertEqual(
            n.ioc.getCollection('JobServer').delete_one({'_id': server}).deleted_count,
            1
        )
        n.ioc.getCollection('ServerHealth').drop()

    def test_serverDeactivateGood(self):
        n = NodeMonitoring()
        server = n.ioc.getCollection('JobServer').insert_one({
            'jobs': 9,
            'os': platform.system().lower(),
            'roles': [],
            'prototypes': [],
            'active': True,
            'activationTime': datetime.datetime.utcnow()
        }).inserted_id
        self.assertTrue(n.deactivateServer(str(server)))
        self.assertEqual(
            n.ioc.getCollection('JobServer').delete_one({'_id': server}).deleted_count,
            1
        )
        n.ioc.getCollection('ServerHealth').drop()

    def test_serverDeactivateFailed(self):
        n = NodeMonitoring()
        server = n.ioc.getCollection('JobServer').insert_one({
            'jobs': 9,
            'os': platform.system().lower(),
            'roles': [],
            'prototypes': [],
            'active': False,
            'activationTime': datetime.datetime.utcnow()
        }).inserted_id
        self.assertEqual(
            n.ioc.getCollection('JobServer').delete_one({'_id': server}).deleted_count,
            1
        )
        self.assertFalse(n.deactivateServer(str(server)))
        n.ioc.getCollection('ServerHealth').drop()

    def test_rescheduleDetectJobsGood(self):
        n = NodeMonitoring()
        server1 = n.ioc.getCollection('JobServer').insert_one({
            'jobs': 9,
            'os': platform.system().lower(),
            'roles': [],
            'prototypes': [],
            'active': True,
            'activationTime': datetime.datetime.utcnow()
        }).inserted_id
        server2 = n.ioc.getCollection('JobServer').insert_one({
            'jobs': 9,
            'os': platform.system().lower(),
            'roles': ['test'],
            'prototypes': ['detect', 'schedule'],
            'active': True,
            'activationTime': datetime.datetime.utcnow()
        }).inserted_id
        source = n.ioc.getCollection('SourceData').insert_one({
            'grease_data': {
                'sourcing': {
                    'server': server1
                },
                'detection': {
                    'server': server1,
                    'start': None,
                    'end': None,
                    'detection': {}
                },
                'scheduling': {
                    'server': None,
                    'start': None,
                    'end': None
                },
                'execution': {
                    'server': None,
                    'assignmentTime': None,
                    'completeTime': None,
                    'returnData': {},
                    'executionSuccess': False,
                    'commandSuccess': False,
                    'failures': 0
                }
            },
            'source': 'test',
            'configuration': 'test',
            'data': {'test': 'ver'},
            'createTime': datetime.datetime.utcnow(),
            'expiry': Deduplication.generate_max_expiry_time(1)
        }).acknowledged
        self.assertTrue(n.deactivateServer(str(server1)))
        self.assertTrue(n.rescheduleDetectJobs(str(server1)))
        self.assertEqual(
            n.ioc.getCollection('JobServer').delete_one({'_id': server1}).deleted_count,
            1
        )
        self.assertEqual(
            n.ioc.getCollection('JobServer').delete_one({'_id': server2}).deleted_count,
            1
        )
        n.ioc.getCollection('SourceData').delete_one({'_id': source})
        n.ioc.getCollection('ServerHealth').drop()

    def test_rescheduleDetectJobsFailed(self):
        n = NodeMonitoring()
        server1 = n.ioc.getCollection('JobServer').insert_one({
            'jobs': 9,
            'os': platform.system().lower(),
            'roles': [],
            'prototypes': [],
            'active': True,
            'activationTime': datetime.datetime.utcnow()
        }).inserted_id
        n.ioc.getCollection('JobServer').update_many(
            {'active': True},
            {
                '$set': {
                    'prototypes': []
                }
            }
        )
        source = n.ioc.getCollection('SourceData').insert_one({
            'grease_data': {
                'sourcing': {
                    'server': server1
                },
                'detection': {
                    'server': server1,
                    'start': None,
                    'end': None,
                    'detection': {}
                },
                'scheduling': {
                    'server': None,
                    'start': None,
                    'end': None
                },
                'execution': {
                    'server': None,
                    'assignmentTime': None,
                    'completeTime': None,
                    'returnData': {},
                    'executionSuccess': False,
                    'commandSuccess': False,
                    'failures': 0
                }
            },
            'source': 'test',
            'configuration': 'test',
            'data': {'test': 'ver'},
            'createTime': datetime.datetime.utcnow(),
            'expiry': Deduplication.generate_max_expiry_time(1)
        }).acknowledged
        self.assertTrue(n.deactivateServer(str(server1)))
        self.assertFalse(n.rescheduleDetectJobs(str(server1)))
        self.assertEqual(
            n.ioc.getCollection('JobServer').delete_one({'_id': server1}).deleted_count,
            1
        )
        n.ioc.getCollection('SourceData').delete_one({'_id': source})
        n.ioc.getCollection('ServerHealth').drop()

    def test_rescheduleScheduleJobsGood(self):
        n = NodeMonitoring()
        server1 = n.ioc.getCollection('JobServer').insert_one({
            'jobs': 9,
            'os': platform.system().lower(),
            'roles': [],
            'prototypes': [],
            'active': True,
            'activationTime': datetime.datetime.utcnow()
        }).inserted_id
        server2 = n.ioc.getCollection('JobServer').insert_one({
            'jobs': 9,
            'os': platform.system().lower(),
            'roles': ['test'],
            'prototypes': ['detect', 'schedule'],
            'active': True,
            'activationTime': datetime.datetime.utcnow()
        }).inserted_id
        config = n.ioc.getCollection('Configuration').insert_one(
            {
                'active': True,
                'type': 'prototype_config',
                "name": "test",
                "job": "help",
                "exe_env": "test",
                "source": "test",
                "logic": {
                    "Regex": [
                        {
                            "field": "url",
                            "pattern": ".*",
                            'variable': True,
                            'variable_name': 'url'
                        }
                    ],
                    'Range': [
                        {
                            'field': 'status_code',
                            'min': 199,
                            'max': 201
                        }
                    ]
                },
                'constants': {
                    'test': 'ver'
                }
            }
        ).inserted_id
        source = n.ioc.getCollection('SourceData').insert_one({
            'grease_data': {
                'sourcing': {
                    'server': server1
                },
                'detection': {
                    'server': server1,
                    'start': None,
                    'end': None,
                    'detection': {}
                },
                'scheduling': {
                    'server': None,
                    'start': None,
                    'end': None
                },
                'execution': {
                    'server': None,
                    'assignmentTime': None,
                    'completeTime': None,
                    'returnData': {},
                    'executionSuccess': False,
                    'commandSuccess': False,
                    'failures': 0
                }
            },
            'source': 'test',
            'configuration': 'test',
            'data': {'test': 'ver'},
            'createTime': datetime.datetime.utcnow(),
            'expiry': Deduplication.generate_max_expiry_time(1)
        }).acknowledged
        self.assertTrue(n.deactivateServer(str(server1)))
        self.assertTrue(n.rescheduleScheduleJobs(str(server1)))
        self.assertEqual(
            n.ioc.getCollection('JobServer').delete_one({'_id': server1}).deleted_count,
            1
        )
        self.assertEqual(
            n.ioc.getCollection('JobServer').delete_one({'_id': server2}).deleted_count,
            1
        )
        n.ioc.getCollection('SourceData').delete_one({'_id': source})
        n.ioc.getCollection('Configuration').delete_one({'_id': config})
        n.ioc.getCollection('ServerHealth').drop()

    def test_rescheduleScheduleJobsFailed(self):
        n = NodeMonitoring()
        server1 = n.ioc.getCollection('JobServer').insert_one({
            'jobs': 9,
            'os': platform.system().lower(),
            'roles': [],
            'prototypes': [],
            'active': True,
            'activationTime': datetime.datetime.utcnow()
        }).inserted_id
        server2 = n.ioc.getCollection('JobServer').insert_one({
            'jobs': 9,
            'os': platform.system().lower(),
            'roles': ['test'],
            'prototypes': [],
            'active': True,
            'activationTime': datetime.datetime.utcnow()
        }).inserted_id
        config = n.ioc.getCollection('Configuration').insert_one(
            {
                'active': True,
                'type': 'prototype_config',
                "name": "test",
                "job": "help",
                "exe_env": "test",
                "source": "test",
                "logic": {
                    "Regex": [
                        {
                            "field": "url",
                            "pattern": ".*",
                            'variable': True,
                            'variable_name': 'url'
                        }
                    ],
                    'Range': [
                        {
                            'field': 'status_code',
                            'min': 199,
                            'max': 201
                        }
                    ]
                },
                'constants': {
                    'test': 'ver'
                }
            }
        ).inserted_id
        n.ioc.getCollection('JobServer').update_many(
            {'active': True},
            {
                '$set': {
                    'prototypes': []
                }
            }
        )
        source = n.ioc.getCollection('SourceData').insert_one({
            'grease_data': {
                'sourcing': {
                    'server': server1
                },
                'detection': {
                    'server': server1,
                    'start': None,
                    'end': None,
                    'detection': {}
                },
                'scheduling': {
                    'server': server1,
                    'start': None,
                    'end': None
                },
                'execution': {
                    'server': None,
                    'assignmentTime': None,
                    'completeTime': None,
                    'returnData': {},
                    'executionSuccess': False,
                    'commandSuccess': False,
                    'failures': 0
                }
            },
            'source': 'test',
            'configuration': 'test',
            'data': {'test': 'ver'},
            'createTime': datetime.datetime.utcnow(),
            'expiry': Deduplication.generate_max_expiry_time(1)
        }).acknowledged
        self.assertTrue(n.deactivateServer(str(server1)))
        self.assertFalse(n.rescheduleScheduleJobs(str(server1)))
        self.assertEqual(
            n.ioc.getCollection('JobServer').delete_one({'_id': server1}).deleted_count,
            1
        )
        self.assertEqual(
            n.ioc.getCollection('JobServer').delete_one({'_id': server2}).deleted_count,
            1
        )
        n.ioc.getCollection('SourceData').delete_one({'_id': source})
        n.ioc.getCollection('Configuration').delete_one({'_id': config})
        n.ioc.getCollection('ServerHealth').drop()

    def test_rescheduleJobsGood(self):
        n = NodeMonitoring()
        p = PrototypeConfig(n.ioc)
        server1 = n.ioc.getCollection('JobServer').insert_one({
            'jobs': 9,
            'os': platform.system().lower(),
            'roles': [],
            'prototypes': [],
            'active': True,
            'activationTime': datetime.datetime.utcnow()
        }).inserted_id
        server2 = n.ioc.getCollection('JobServer').insert_one({
            'jobs': 9,
            'os': platform.system().lower(),
            'roles': ['test'],
            'prototypes': ['detect', 'schedule'],
            'active': True,
            'activationTime': datetime.datetime.utcnow()
        }).inserted_id
        config = n.ioc.getCollection('Configuration').insert_one(
            {
                'active': True,
                'type': 'prototype_config',
                "name": "test",
                "job": "help",
                "exe_env": "test",
                "source": "test",
                "logic": {
                    "Regex": [
                        {
                            "field": "url",
                            "pattern": ".*",
                            'variable': True,
                            'variable_name': 'url'
                        }
                    ],
                    'Range': [
                        {
                            'field': 'status_code',
                            'min': 199,
                            'max': 201
                        }
                    ]
                },
                'constants': {
                    'test': 'ver'
                }
            }
        ).inserted_id
        p.load(reloadConf=True)
        source = n.ioc.getCollection('SourceData').insert_one({
            'grease_data': {
                'sourcing': {
                    'server': server1
                },
                'detection': {
                    'server': server1,
                    'start': datetime.datetime.utcnow(),
                    'end': datetime.datetime.utcnow(),
                    'detection': {}
                },
                'scheduling': {
                    'server': server1,
                    'start': datetime.datetime.utcnow(),
                    'end': datetime.datetime.utcnow(),
                },
                'execution': {
                    'server': server1,
                    'assignmentTime': datetime.datetime.utcnow(),
                    'completeTime': None,
                    'returnData': {},
                    'executionSuccess': False,
                    'commandSuccess': False,
                    'failures': 0
                }
            },
            'source': 'test',
            'configuration': 'test',
            'createTime': datetime.datetime.utcnow(),
            'expiry': Deduplication.generate_max_expiry_time(1)
        }).inserted_id
        self.assertTrue(n.deactivateServer(str(server1)))
        self.assertTrue(n.rescheduleJobs(str(server1)))
        self.assertTrue(n.ioc.getCollection('SourceData').find({
            '_id': source,
            'grease_data.execution.server': server2
        }).count())
        self.assertEqual(
            n.ioc.getCollection('JobServer').delete_one({'_id': server1}).deleted_count,
            1
        )
        self.assertEqual(
            n.ioc.getCollection('JobServer').delete_one({'_id': server2}).deleted_count,
            1
        )
        n.ioc.getCollection('SourceData').delete_one({'_id': source})
        n.ioc.getCollection('Configuration').drop()
        n.ioc.getCollection('ServerHealth').drop()
        p.load(reloadConf=True)

    def test_rescheduleJobsFailed(self):
        n = NodeMonitoring()
        p = PrototypeConfig(n.ioc)
        server1 = n.ioc.getCollection('JobServer').insert_one({
            'jobs': 9,
            'os': platform.system().lower(),
            'roles': [],
            'prototypes': [],
            'active': True,
            'activationTime': datetime.datetime.utcnow()
        }).inserted_id
        server2 = n.ioc.getCollection('JobServer').insert_one({
            'jobs': 9,
            'os': platform.system().lower(),
            'roles': ['test1'],
            'prototypes': ['detect', 'schedule'],
            'active': True,
            'activationTime': datetime.datetime.utcnow()
        }).inserted_id
        config = n.ioc.getCollection('Configuration').insert_one(
            {
                'active': True,
                'type': 'prototype_config',
                "name": "test",
                "job": "help",
                "exe_env": "test",
                "source": "test",
                "logic": {
                    "Regex": [
                        {
                            "field": "url",
                            "pattern": ".*",
                            'variable': True,
                            'variable_name': 'url'
                        }
                    ],
                    'Range': [
                        {
                            'field': 'status_code',
                            'min': 199,
                            'max': 201
                        }
                    ]
                },
                'constants': {
                    'test': 'ver'
                }
            }
        ).inserted_id
        p.load(reloadConf=True)
        source = n.ioc.getCollection('SourceData').insert_one({
            'grease_data': {
                'sourcing': {
                    'server': server1
                },
                'detection': {
                    'server': server1,
                    'start': datetime.datetime.utcnow(),
                    'end': datetime.datetime.utcnow(),
                    'detection': {}
                },
                'scheduling': {
                    'server': server1,
                    'start': datetime.datetime.utcnow(),
                    'end': datetime.datetime.utcnow(),
                },
                'execution': {
                    'server': server1,
                    'assignmentTime': datetime.datetime.utcnow(),
                    'completeTime': None,
                    'returnData': {},
                    'executionSuccess': False,
                    'commandSuccess': False,
                    'failures': 0
                }
            },
            'source': 'test',
            'configuration': 'test',
            'createTime': datetime.datetime.utcnow(),
            'expiry': Deduplication.generate_max_expiry_time(1)
        }).inserted_id
        self.assertTrue(n.deactivateServer(str(server1)))
        self.assertFalse(n.rescheduleJobs(str(server1)))
        self.assertFalse(n.ioc.getCollection('SourceData').find({
            '_id': source,
            'grease_data.execution.server': server2
        }).count())
        self.assertEqual(
            n.ioc.getCollection('JobServer').delete_one({'_id': server1}).deleted_count,
            1
        )
        self.assertEqual(
            n.ioc.getCollection('JobServer').delete_one({'_id': server2}).deleted_count,
            1
        )
        n.ioc.getCollection('SourceData').delete_one({'_id': source})
        n.ioc.getCollection('Configuration').drop()
        n.ioc.getCollection('ServerHealth').drop()
        p.load(reloadConf=True)
